@@ -223,18 +223,6 @@ int vm_run(META *m, VM *vm)
 			/* See if this term needs fixing up */
 			if ((av->flags & AV_UNRESOLVED) && av->i) {
 
-				/* Yes, so first fixup the list reference using
-				 * the opcode. If AVR_HAVE_LIST is not set, the
-				 * capitalization of the reference (recorded by
-				 * compiler in 'list') determines what happens
-				 * next, together with the opcode. Uppercase
-				 * references make us use the opcode's default,
-				 * lowercase references make us use the
-				 * opcode's inverted default. */
-
-				if ((av->ref.have & AVR_HAVE_LIST) == 0)
-				    av->ref.list ^= OP_USEREP(i->op, fixcnt);
-
 				/* See if we want the first or last instance */
 				if (av->ref.inst == AVR_INST_FIRST) {
 
@@ -288,16 +276,19 @@ int vm_run(META *m, VM *vm)
 			break;
 		  case OP_ACCEPT:
 			for(av = vm->head[VM_REP]; av; av = av->next) {
-				if (av->i->spc->nr == C_DS_RAD_PKT && av->i->nr == C_DI_CODE) 
+				if (av->i->spc->nr == C_DS_RAD_PKT && 
+				    av->i->nr == C_DI_CODE) 
 					av->l = C_DV_CODE_ACCEPT;
 			}
 			ret = VM_HALTED; 
 			break;
 		  case OP_REJECT:
 			for(av = vm->head[VM_REP]; av; av = av->next) {
-				if (av->i->spc->nr == C_DS_RAD_PKT && av->i->nr == C_DI_CODE) 
+				if (av->i->spc->nr == C_DS_RAD_PKT && 
+				    av->i->nr == C_DI_CODE) 
 					av->l = C_DV_CODE_REJECT;
-				if (av->i->spc->nr != C_DS_RAD_ATR || av->i->keeprej) 
+				if (av->i->spc->nr != C_DS_RAD_ATR || 
+				    av->i->keeprej) 
 					continue;
 				av->flags |= AV_DEL;
 			}
@@ -305,9 +296,11 @@ int vm_run(META *m, VM *vm)
 			break;
 		  case OP_ACCTRESP:
 			for(av = vm->head[VM_REP]; av; av = av->next) {
-				if (av->i->spc->nr == C_DS_RAD_PKT && av->i->nr == C_DI_CODE) 
+				if (av->i->spc->nr == C_DS_RAD_PKT && 
+				    av->i->nr == C_DI_CODE) 
 					av->l = C_DV_CODE_ACCTRESP;
-				if (av->i->spc->nr != C_DS_RAD_ATR || av->i->keepacct) 
+				if (av->i->spc->nr != C_DS_RAD_ATR || 
+				    av->i->keepacct) 
 					continue;
 				av->flags |= AV_DEL;
 			}
@@ -372,21 +365,16 @@ int vm_run(META *m, VM *vm)
 			   the right to determine how we should encode the
 			   request or reply list. This moves the space search 
 			   to the compiler. The attribute reference's list 
-			   specifies the list to be encoded; the reply list 
-			   being the default. */
+			   specifier indicates the list to be encoded. */
 			meta_freeavdata(&R); R.p = 0; R.l = 0;
 			if (!R.i) break;
 
-			/* Fixup list; reply is default. */
-			if ((R.ref.have & AVR_HAVE_LIST) == 0)
-			    R.ref.list ^= AVR_LIST_REP;
-
 			/* Build a tree from the encapsulating space */
 			avlst = 0;
 			meta_buildtree(vm->head[R.ref.list], &avlst, R.i->spc);
 			if (!avlst) break;
 			if (msg_thresh[F_SEND] >= L_DEBUG) {
-				msg(F_SEND, L_DEBUG, "OP_DICTENC: tree for %s list from %s:\n", R.ref.list ? "REP" : "REQ", R.i->spc->name);
+				msg(F_SEND, L_DEBUG, "OP_DICTENC: tree for %s list from %s:\n", R.ref.list == AVR_LIST_REP ? "REP" : "REQ", R.i->spc->name);
 				meta_printavlist(m, avlst, 0);
 			}
 			/* Now encode it */
@@ -551,15 +539,16 @@ int vm_run(META *m, VM *vm)
 			   Also, we use the reference's list to determine the 
 			   destination list, and the reference's instance spec
 			   to determine where the decoded list should end up. */
-			/* meta_decode normally assumes the source data to be 
-			   persistent. So, neither L.flags & AV_FREE_P must be
-			   set, nor, if L.sub is present, L.sub->flags &
-			   AV_FREE_P. That means only strings that are constant
-			   (coming from the expression) or strings that come
-			   from the original request (not module responses!).
-			   In all other cases, we pass a 'copystrings' flag to
-			   meta_decode, causing it to create a copy of all
-			   strings, setting their AV_FREE_P flags. */
+			/* Note that meta_decode normally assumes the source
+			   data to be persistent. So, neither L.flags &
+			   AV_FREE_P must be set, nor, if L.sub is present,
+			   L.sub->flags & AV_FREE_P. That means only strings
+			   that are constant (coming from the expression) or
+			   strings that come from the original request (not
+			   module responses!).  In all other cases, we pass a
+			   'copystrings' flag to meta_decode, causing it to
+			   create a copy of all strings, setting their
+			   AV_FREE_P flags. */
 
 			av = 0;
 			if (!R.i || !L.l ||
@@ -914,10 +903,6 @@ int vm_run(META *m, VM *vm)
 
 		  case OP_DELALLAV: if (sp < 1) { ret = VM_UNDERFLOW; break; } 
 
-			/* Fixup list; reply is default. */
-			if ((R.ref.have & AVR_HAVE_LIST) == 0)
-			    R.ref.list ^= AVR_LIST_REP;
-
 			/* Already free right operand's data if any */
 			meta_freeavdata(&R); R.p = 0; R.l = 0;
 
