@@ -43,21 +43,6 @@ char jobs_id[] = "JOBS - Copyright (C) 2001 Emile van Bergen.";
  */
 
 
-static META_AV *meta_addnewav(META_AV **head, META_AV **tail, 
-			      META_AV *rel, int before, 
-			      META_ITEM *i, char *p, META_ORD l)
-{
-	META_AV *av;
-
-	av = (META_AV *)malloc(sizeof(META_AV)); if (!av) return 0;
-	memset(av, 0, sizeof(META_AV));
-	av->i = i; av->p = p; av->l = l; 
-
-	meta_addav(head, tail, rel, before, av);
-	return av;
-}
-
-
 /* Process list of pairs received from modules according to ACL and ACL's and
  * item's list/inst/op specifiers. */
 
@@ -71,7 +56,7 @@ static void job_process_recvlist(META_AV **heads, META_AV **tails, META_AV *acl,
     /* Walk received list. We remember the next item separately because we move
      * av to a different list inside the loop. */
 
-    msg(F_PROC, L_NOTICE, "job_process_recvlist:\n");
+    msg(F_PROC, L_DEBUG, "job_process_recvlist:\n");
     for(av = *recvhead; av; av = nextav) {
 	nextav = av->next;
 	meta_remav(recvhead, recvtail, av);
@@ -98,7 +83,7 @@ static void job_process_recvlist(META_AV **heads, META_AV **tails, META_AV *acl,
 		  av->ref.have |= AVR_HAVE_OP,   av->ref.op   = auxav->ref.op;
 	}
 	if (!keep) { 
-	    msg(F_PROC, L_NOTICE, "  '%s' not in receive ACL - ignored\n", 
+	    msg(F_PROC, L_DEBUG, "  '%s' not in receive ACL - ignored\n", 
 		av->i->name);
 	    meta_freeavdata(av); free(av); 
 	    continue; 
@@ -117,7 +102,7 @@ static void job_process_recvlist(META_AV **heads, META_AV **tails, META_AV *acl,
 	if (av->ref.op == AVR_OP_ADD) {
 	    meta_addav(&heads[av->ref.list], &tails[av->ref.list], 0,
 		       av->ref.inst == AVR_INST_FIRST, av);
-	    msg(F_PROC, L_NOTICE, "  '%s' added at %s of %s list\n", 
+	    msg(F_PROC, L_DEBUG, "  '%s' added at %s of %s list\n", 
 		name, headtail, reqrep);
 	    continue;
 	}
@@ -138,14 +123,14 @@ static void job_process_recvlist(META_AV **heads, META_AV **tails, META_AV *acl,
 
 	if (!auxav) {
 	    if (av->ref.op == AVR_OP_DEL) { 
-		msg(F_PROC, L_NOTICE, "  '%s' not on %s list - ignored\n", 
+		msg(F_PROC, L_DEBUG, "  '%s' not on %s list - ignored\n", 
 		    name, reqrep);
 		meta_freeavdata(av); free(av);
 		continue;
 	    }
 	    meta_addav(&heads[av->ref.list], &tails[av->ref.list], 0,
 		       av->ref.inst == AVR_INST_FIRST, av);
-	    msg(F_PROC, L_NOTICE, "  '%s' added at %s of %s list\n", 
+	    msg(F_PROC, L_DEBUG, "  '%s' added at %s of %s list\n", 
 		name, headtail, reqrep);
 	    continue;
 	}
@@ -159,14 +144,14 @@ static void job_process_recvlist(META_AV **heads, META_AV **tails, META_AV *acl,
 	    auxav->p = av->p;
 	    auxav->flags = av->flags;	/* important to copy AV_FREE_P */
 	    free(av);
-	    msg(F_PROC, L_NOTICE, "  '%s' replaced %s instance on %s list\n", 
+	    msg(F_PROC, L_DEBUG, "  '%s' replaced %s instance on %s list\n", 
 		name, firstlast, reqrep);
 	    continue;
 	}
 
 	if (av->ref.op == AVR_OP_DEL) {
 	    auxav->flags |= AV_DEL;
-	    msg(F_PROC, L_NOTICE, "  '%s' deleted %s instance on %s list\n", 
+	    msg(F_PROC, L_DEBUG, "  '%s' deleted %s instance on %s list\n", 
 		name, firstlast, reqrep);
 	}
 
@@ -176,6 +161,79 @@ static void job_process_recvlist(META_AV **heads, META_AV **tails, META_AV *acl,
 }
 
 
+/* Obtains list of AV pairs to send from request (and reply) list according
+ * to ACL. Copies pairs, clears AV_FREE_P flags so that meta_freeavlist can
+ * be used safely; data owned by source list stays that way. */
+
+static void job_get_sendlist(META_AV **heads, META_AV **tails, META_AV *acl,
+			     META_AV **sendhead, META_AV **sendtail)
+{
+    META_AV *av, *aclav;
+
+    /* Walk ACL. Generally more efficient than walking source lists.
+     * Disadvantage is that in order for negative ACLs to work, you
+     * must put negative items after the positive ones. We'll fix that later. */
+
+    for(aclav = acl; aclav; aclav = aclav->next) {
+
+	/* Code should ideally be shared with vm_run (esp. fixup). We should
+	 * really have a 'getpairbyref' and a 'processpairbyref' in metaops.c
+	 */
+
+	if ((aclav->ref.have & AVR_HAVE_INST) == 0) {
+
+	    /* Take all instances from referenced list */
+
+	    for(av = heads[aclav->ref.list]; av; av = av->next) {
+		if ((aclav->i->nr != C_DI_ANY && av->i != aclav->i) ||
+		    av->flags & AV_DEL) continue;
+		meta_addnewav(sendhead, sendtail, 0, 0, av->i, av->p, av->l);
+	    }
+	    continue;
+	}
+	if (aclav->ref.inst == AVR_INST_FIRST) { 
+	    
+	    /* Take first match forwards. */
+	    /* Note: we don't treat the 'any' attribute on the ACL in any
+	     * special way. It would be nice to take the first match from 'any'
+	     * in this case. Will only work if we indeed switch to source list
+	     * driven loop. */
+	    
+	    for(av = heads[acl->ref.list]; av && (av->flags & AV_DEL || av->i
+		    != aclav->i); av = av->next);
+
+	    if (av) meta_addnewav(sendhead, sendtail, 0,0, av->i, av->p, av->l);
+	    continue;
+	}
+	if (aclav->ref.inst == AVR_INST_LAST) { 
+
+	    /* Take first match backwards */
+	    /* Note: we don't treat the 'any' attribute on the ACL in any
+	     * special way. It would be nice to take the first match from 'any'
+	     * in this case. Will only work if we indeed switch to source list
+	     * driven loop. */
+	    
+	    for(av = heads[acl->ref.list]; av && (av->flags & AV_DEL || av->i
+		    != aclav->i); av = av->next);
+	    
+	    for(av = tails[acl->ref.list];
+		av && (av->flags & AV_DEL || av->i != aclav->i);
+		av = av->prev);
+
+	    if (av) meta_addnewav(sendhead, sendtail, 0,0, av->i, av->p, av->l);
+	    continue;
+	}
+
+	/* Negative. Set delete flag on all referenced items that are already
+	 * on the send list. Does not regard list spec on ACL, nor wildcards. */
+
+	if (aclav->ref.inst != AVR_INST_NO) continue;
+	for(av = *sendhead; av; av = av->next) {
+	    if (av->i == aclav->i) av->flags |= AV_DEL;
+	}
+    }
+}
+
 
 static JOB *job_new_common(JOB *j, CONF *c, 
 			   META_AV *reqh, META_AV *reqt, META_AV *reqcur)
@@ -463,7 +521,7 @@ IFACE *job_run(CONF *c, JOB *j)
 			meta_printavlist(c->m, j->vm->head[VM_REP], 0);
 
 		j->pktlen = meta_avtomsg(c->m, j->vm->head[VM_REP],
-					 j->pkt, C_MAX_MSGSIZE, 0, 0,
+					 j->pkt, C_MAX_MSGSIZE, 0,
 					 C_BINMSG_REPMAGIC);
 
 		if (msg_thresh[F_SEND] >= L_DEBUG) hexdump(j->pkt, j->pktlen);
@@ -602,7 +660,7 @@ int job_toiface(IFACE *i, JOB *j, time_t t)
 void job_tochan(CHAN *ch, JOB *j, time_t t)
 {
 	static char reqmsg[C_MAX_MSGSIZE], jobticketbuf[16];
-	META_AV av;
+	META_AV jobav, *sendhead, *sendtail;
 	ssize_t msglen;
 	PROC *p;
 
@@ -619,24 +677,31 @@ void job_tochan(CHAN *ch, JOB *j, time_t t)
 
 	/* Add A/V pair for ch->job_w if we use job tickets */
 	if (ch->iface->jobticket) {
-		memset(&av, 0, sizeof(av));
-		av.i = ch->iface->jobticket;
-		av.l = ch->job_w;
-		if (av.i->val_type == MT_STRING) {
-			av.l = meta_ordtoa(jobticketbuf, sizeof(jobticketbuf), 
+		memset(&jobav, 0, sizeof(jobav));
+		jobav.i = ch->iface->jobticket;
+		jobav.l = ch->job_w;
+		if (jobav.i->val_type == MT_STRING) {
+			jobav.l = meta_ordtoa(jobticketbuf, sizeof(jobticketbuf), 
 					   0, 10, ch->job_w);
-			av.p = jobticketbuf;
+			jobav.p = jobticketbuf;
 		}
-		av.prev = j->vm->tail[VM_REQ];
-		if (j->vm->tail[VM_REQ]) j->vm->tail[VM_REQ]->next = &av;
-		else j->vm->head[VM_REQ] = &av;
+		jobav.prev = j->vm->tail[VM_REQ];
+		if (j->vm->tail[VM_REQ]) j->vm->tail[VM_REQ]->next = &jobav;
+		else j->vm->head[VM_REQ] = &jobav;
+	}
+
+	/* Obtain send list using ACL (full request list if none) */
+	sendhead = j->vm->head[VM_REQ]; sendtail = 0;
+	if (ch->iface->sendacl) {
+		sendhead = 0;
+		job_get_sendlist(j->vm->head, j->vm->tail, ch->iface->sendacl,
+				 &sendhead, &sendtail);
 	}
 
 	/* Create the message from the request list. */
-	msglen = meta_avtomsg(ch->iface->c->m, j->vm->head[VM_REQ],
+	msglen = meta_avtomsg(ch->iface->c->m, sendhead,
 			      reqmsg, C_MAX_MSGSIZE, 
-			      ch->iface->flags, ch->iface->sendacl,
-			      C_BINMSG_REQMAGIC);
+			      ch->iface->flags, C_BINMSG_REQMAGIC);
 	if (msg_thresh[F_PROC] >= L_DEBUG) {
 		msg(F_PROC, L_DEBUG, "Message to %s's subprocess %s, pid %d:\n",
 		    ch->iface->name, ch->proc->cmd, p->pid);
@@ -644,17 +709,22 @@ void job_tochan(CHAN *ch, JOB *j, time_t t)
 		else hexdump(reqmsg, msglen);
 	}
 
-	/* Remove temporary job ticket pair from list tail */
+	/* Free temporary send list, if we created any using job_get_sendlist.
+	 * Excuse the ugly reuse of sendtail as a flag. */
+	if (sendtail) meta_freeavlist(sendhead);
+
+	/* Remove temporary job ticket pair from request list tail again */
 	if (ch->iface->jobticket) {
-		if (av.prev) av.prev->next = 0;
+		if (jobav.prev) jobav.prev->next = 0;
 		else j->vm->head[VM_REQ] = 0;
-		j->vm->tail[VM_REQ] = av.prev;
+		j->vm->tail[VM_REQ] = jobav.prev;
 	}
 
 	/* Put the message in the channel's proc's ring. I'd agree we copy 
 	   things around a lot - avtomsg could be done straight to the ring.
 	   The only times meta_avtomsg is used right now is here and in
-	   printav(), which would also benefit from that... */
+	   printav(), which would also benefit from that... Oh well, let's
+	   wait till we port evblib's zero copy ring buffers */
 	ring_put(p->w, reqmsg, msglen);
 
 	/* Set expiry time in job */
